@@ -65,6 +65,7 @@ let fork_and_exec cmd args =
 *)
 
 let setuid uid = ()
+let setgid gid = ()
 let set_close_on_exec fd = ()
 let set_signal signal f = Sys.set_signal signal f
   
@@ -73,6 +74,7 @@ let max_filedescs = 50
 let chroot _ = ()
 
 external set_nonblock: Unix.file_descr -> unit = "ml_set_nonblock"
+external set_console_title : string -> unit = "os_set_console_title"
 
 let mini (x: int) y = if x < y then x else y
 
