@@ -963,15 +963,14 @@ let http_handler o t r =
   let user = if r.options.login = "" then (admin_user ()).CommonTypes.user_name else r.options.login in
   if not (valid_password user r.options.passwd) || (r.get_url.Url.short_file = "logout") then begin
       clear_page buf;
-      http_file_type := TXT;
-      let _, error_text_long, header = Http_server.error_page "401" "" ""
+      http_file_type := HTM;
+      let _, error_text_long, head = Http_server.error_page "401" "" ""
 	(Ip.to_string (TcpBufferedSocket.my_ip r.sock))
 	(string_of_int !!http_port) None in
       Buffer.add_string buf error_text_long;
-      r.reply_head <- header;
-      r.reply_headers <- [
-        "Connection", "close";
-        "WWW-Authenticate", Printf.sprintf "Basic realm=\"%s\"" !!http_realm]
+      r.reply_head <- head;
+      http_add_html_header r;
+      add_reply_header r "WWW-Authenticate" (Printf.sprintf "Basic realm=\"%s\"" !!http_realm);
     end
   else
     begin
@@ -1520,14 +1519,17 @@ let http_handler o t r =
         | s ->  http_send_bin_pictures r buf (String.lowercase s)
       with
       | Not_found ->
-	  let _, error_text_long, header = Http_server.error_page "404" "" ""
+	  let _, error_text_long, head = Http_server.error_page "404" "" ""
 			(Ip.to_string (TcpBufferedSocket.my_ip r.sock))
 			(string_of_int !!http_port)
 			(Some (Url_not_found r.get_url.Url.full_file)) in
-	  r.reply_head <- header;
+	  r.reply_head <- head;
+          http_add_html_header r;
 	  Buffer.add_string buf error_text_long
       | e ->
-          Printf.bprintf buf "\nException %s\n" (Printexc2.to_string e);
+          http_add_text_header r TEXTS;
+          Printf.bprintf buf "%sException %s\n"
+            (if Buffer.length buf = 0 then "" else "\n") (Printexc2.to_string e);
           r.reply_stream <- None
     end;
 
