@@ -108,14 +108,32 @@ let start_interfaces () =
             (Printexc2.to_string e);
     end;
 
-  ignore (find_port  "telnet server" !!telnet_bind_addr
-      telnet_port DriverControlers.telnet_handler);
+  if !!telnet_port <> 0 then begin 
+      try
+        ignore (find_port "telnet server" !!telnet_bind_addr
+          telnet_port DriverControlers.telnet_handler);
+      with e ->
+          lprintf_nl (_b "Exception %s while starting Telnet interface")
+            (Printexc2.to_string e);
+    end;
 
-  gui_server_sock := find_port "gui server"  !!gui_bind_addr
-    gui_port gui_handler;
-  if !!gift_port <> 0 then
-    ignore (find_port "gift server"  !!gui_bind_addr
-        gift_port gift_handler);
+  if !!gui_port <> 0 then begin 
+      try
+        ignore (find_port "gui server" !!gui_bind_addr
+          gui_port gui_handler);
+      with e ->
+          lprintf_nl (_b "Exception %s while starting GUI interface")
+            (Printexc2.to_string e);
+    end;
+
+  if !!gift_port <> 0 then begin
+      try
+        ignore (find_port "gift server" !!gui_bind_addr
+          gift_port gift_handler);
+      with e ->
+          lprintf_nl (_b "Exception %s while starting GUI interface")
+            (Printexc2.to_string e);
+    end;
 
   add_infinite_option_timer update_gui_delay DriverInterface.update_gui_info;
   add_infinite_timer 1. second_timer
@@ -400,7 +418,7 @@ or getting a binary compiled with glibc %s.\n\n")
 	end
       end
   );
-  if not !Charset.conversion_enabled then
+  if not !Charset.Locale.conversion_enabled then
     lprintf_nl (_b "Self-test failed, charset conversion disabled.");
 
   load_config ();
@@ -440,10 +458,33 @@ or getting a binary compiled with glibc %s.\n\n")
   lprintf_nl (_b "Check http://www.mldonkey.org for updates");
   networks_iter (fun r -> network_load_complex_options r);
   lprintf_nl (_b "enabling networks: ");
+  if (upnp_port_forwarding ()) then
+    UpnpClient.init_maps ();
+    let add_upnp_port p s=
+      lprintf_nl "using port %d (%s)" p s;
+      if ((upnp_port_forwarding ())) then
+        match String2.split_simplify s ' ' with
+          | [ "client_port" ; tcpudp   ]
+          | [ "tracker_port" ; tcpudp   ]
+          | [ "dht_port" ; tcpudp      ]
+          | [ "overnet_port" ; tcpudp  ]
+          | [ "kademlia_port" ; tcpudp ] ->
+          if (String2.contains tcpudp "TCP") then 
+            begin
+            UpnpClient.maps_add_item 1 p p 1 "" ;
+            lprintf_nl "add upnp port forwarding %d TCP" p;
+            end;
+          if (String2.contains tcpudp "UDP") then
+            begin
+            UpnpClient.maps_add_item 1 p p 0 "" ;
+            lprintf_nl "add upnp port forwarding %d UDP" p;
+            end
+          | _ -> ()
+  in
   networks_iter (fun r ->
       lprintf_nl (_b "---- enabling %s ----") r.network_name;
       network_enable r;
-      List.iter (fun (p,s) -> if p <> 0 then lprintf_nl "using port %d (%s)" p s) (network_ports r);
+      List.iter (fun (p,s) -> if p <> 0 then add_upnp_port p s) (network_ports r);
 (* are there drawbacks to start recover_temp unconditionally here ? *)
       if !!recover_temp_on_startup then
         network_recover_temp r;
@@ -461,6 +502,8 @@ or getting a binary compiled with glibc %s.\n\n")
 	  end);
   if not !found then lprintf (_b "none");
   lprint_newline ();
+  if (upnp_port_forwarding ()) then
+    UpnpClient.job_start ();
   networks_iter_all (fun n -> network_update_options n);
   CommonOptions.start_running_plugins := true;
   CommonInteractive.force_download_quotas ();
@@ -516,14 +559,16 @@ or getting a binary compiled with glibc %s.\n\n")
   Options.prune_file downloads_ini;
   Options.prune_file users_ini;
   add_timer 1. (fun _ -> try CommonWeb.load_web_infos true false with _ -> ());
-  lprintf_nl  (_b "To command: telnet %s %d")
+  if !!telnet_port <> 0 then lprintf_nl  (_b "To command: telnet %s %d")
 	(if !!telnet_bind_addr = Ip.any then "127.0.0.1"
 		else Ip.to_string !!telnet_bind_addr)  !!telnet_port;
-  lprintf_nl  (_b "Or with browser: http://%s:%d")
+  if !!http_port <> 0 then begin
+    lprintf_nl  (_b "Or with browser: http://%s:%d")
 	(if !!http_bind_addr = Ip.any then "127.0.0.1"
 		else Ip.to_string !!http_bind_addr)  !!http_port;
-  lprintf_nl  (_b "For a GUI check out http://sancho-gui.sourceforge.net");
-  lprintf_nl  (_b "Connect to IP %s, port %d")
+    lprintf_nl  (_b "For a GUI check out http://sancho-gui.sourceforge.net")
+  end;
+  if !!gui_port <> 0 then lprintf_nl  (_b "Connect to IP %s, port %d")
 	(if !!gui_bind_addr = Ip.any then "127.0.0.1"
 		else Ip.to_string !!gui_bind_addr)  !!gui_port;
   lprintf_nl  (_b "If you connect from a remote machine adjust allowed_ips");
@@ -545,6 +590,28 @@ or getting a binary compiled with glibc %s.\n\n")
             );
   );
 
+  if !!run_as_group <> "" then begin
+      try
+        let new_gr = Unix.getgrnam !!run_as_group in
+        MlUnix.setgid new_gr.Unix.gr_gid;
+        let gr = Unix.getgrgid (Unix.getgid()) in
+        lprintf_nl (_b "mldonkey is now running as group %s") gr.Unix.gr_name;
+      with e ->
+          lprintf_nl (_b "Exception %s trying to set group_gid [%s]")
+          (Printexc2.to_string e) !!run_as_group;
+          exit 67
+    end;
+
+  if !!run_as_groupgid <> 0 then begin
+      try
+        MlUnix.setgid !!run_as_groupgid;
+        lprintf_nl (_b "mldonkey is now running as gid %d")  !!run_as_groupgid;
+      with e ->
+          lprintf_nl (_b "Exception %s trying to set group_gid [%d]")
+          (Printexc2.to_string e) !!run_as_groupgid;
+          exit 67
+    end;
+
   if !!run_as_user <> "" then begin
       try
         let new_pw = Unix.getpwnam !!run_as_user in
