@@ -456,6 +456,7 @@ let value_reader gui t =
     | GiftServerStats _ -> assert false
     | Search s -> ()
     | Version v -> ()
+    | Stats (_, _) -> ()
         
   with e ->
       Printf2.lprintf "Exception %s in reader\n" (Printexc2.to_string e)
@@ -540,11 +541,6 @@ let core_menu gui on_quit =
       kill_core#add_accelerator ~group:accel_menu
         ~modi:[`CONTROL] ~flags:[`VISIBLE] GdkKeysyms._k;
     end;
-  let im =
-    GMenu.image_menu_item ~label:!M.mW_me_im ~use_mnemonic:true
-      ~image:(GMisc.image ~pixbuf:(A.get_icon ~icon:M.icon_menu_im ~size:A.SMALL ()) ())
-      ~packing:menu#add ()
-  in
   let settings =
     GMenu.image_menu_item ~label:!M.mW_me_settings ~use_mnemonic:true
       ~image:(GMisc.image ~pixbuf:(A.get_icon ~icon:M.icon_menu_settings ~size:A.SMALL ()) ())
@@ -555,17 +551,12 @@ let core_menu gui on_quit =
       ~image:(GMisc.image ~pixbuf:(A.get_icon ~icon:M.icon_menu_quit ~size:A.SMALL ()) ())
       ~packing:menu#add ()
   in
-  ignore (im#connect#activate 
-    (fun _ -> GuiWindow.display_im gui ()
-  ));
   ignore (settings#connect#activate 
     (fun _ -> GuiWindow.display_settings gui value_reader ()
   ));
   ignore (quit#connect#activate 
     (fun _ -> on_quit ()
   ));
-  im#add_accelerator ~group:accel_menu
-    ~modi:[`CONTROL] ~flags:[`VISIBLE] GdkKeysyms._i;
   settings#add_accelerator ~group:accel_menu
     ~modi:[`CONTROL] ~flags:[`VISIBLE] GdkKeysyms._o;
   quit#add_accelerator ~group:accel_menu
@@ -628,11 +619,6 @@ let tray_menu gui on_quit =
       ~image:(GMisc.image ~pixbuf:(A.get_icon ~icon:M.icon_menu_interfaces ~size:A.SMALL ()) ())
       ~packing:menu#add ()
   in
-  let im =
-    GMenu.image_menu_item ~label:!M.mW_me_im ~use_mnemonic:true
-      ~image:(GMisc.image ~pixbuf:(A.get_icon ~icon:M.icon_menu_im ~size:A.SMALL ()) ())
-      ~packing:menu#add ()
-  in
   let settings =
     GMenu.image_menu_item ~label:!M.mW_me_settings ~use_mnemonic:true
       ~image:(GMisc.image ~pixbuf:(A.get_icon ~icon:M.icon_menu_settings ~size:A.SMALL ()) ())
@@ -648,9 +634,6 @@ let tray_menu gui on_quit =
        gui.window#show ();
        G.tray.destroy_tray ();
   ));
-  ignore (im#connect#activate 
-    (fun _ -> GuiWindow.display_im gui ()
-  ));
   ignore (settings#connect#activate 
     (fun _ -> GuiWindow.display_settings gui value_reader ()
   ));
@@ -682,15 +665,15 @@ let splash_timerID = ref (GMain.Timeout.add ~ms:2000 ~callback:(fun _ -> true))
 
 let splash_screen_window gui =
   let scrwidth = Gdk.Screen.width () in
-  let colormap = Gdk.Color.get_system_colormap () in
-  let size = scrwidth / 3 in
+  let _colormap = Gdk.Color.get_system_colormap () in
+  let _size = scrwidth / 3 in
   let splash_screen_pb = A.get_icon ~icon:M.icon_splash_screen ~size:A.LARGE () in
   let width = GdkPixbuf.get_width splash_screen_pb in
   let height = GdkPixbuf.get_height splash_screen_pb in
   let (splash_screen_pix , _ ) = GdkPixbuf.create_pixmap splash_screen_pb in
   let wpix = `PIXMAP splash_screen_pix in
   let window =
-    GWindow.window ~position:`CENTER_ALWAYS ~kind:`POPUP
+    GWindow.window ~position:`CENTER_ALWAYS ~type_hint:`SPLASHSCREEN
       ~width ~height ()
   in
   Widget.realize window#as_widget ;
@@ -718,7 +701,7 @@ let splash_screen_window gui =
   in
   status#set_use_markup true;
 
-  let label =
+  let _label =
     GMisc.label ~text:(Printf.sprintf "v. %s" Autoconf.current_version)
       ~xpad:10 ~ypad:10 ~xalign:0. ~yalign:1.
       ~packing:(vbox#pack ~fill:true ~expand:false) ()
@@ -881,6 +864,11 @@ let main () =
     if !G.is_docked then G.tray.destroy_tray ()
   );
 
+  MlUnix.set_signal  Sys.sigint
+    (Sys.Signal_handle (fun _ -> CommonGlobals.exit_properly 0));
+
+  MlUnix.set_signal  Sys.sigterm
+    (Sys.Signal_handle (fun _ -> CommonGlobals.exit_properly 0));
 
   (************ Some hooks ***************)
 
