@@ -548,7 +548,7 @@ let iM_me_settings = ref ""
 let iM_me_quit = ref ""
 let iM_me_remove = ref ""
 let iM_me_join_room = ref ""
-let iM_me_prefered_rooms = ref ""
+let iM_me_preferred_rooms = ref ""
 let iM_lb_new_account = ref ""
 let im_lb_room_name = ref ""
 let iM_lb_join_room = ref ""
@@ -1014,7 +1014,7 @@ let load_messages () =
   iM_me_quit := _s "Quit";
   iM_me_remove := _s "Remove";
   iM_me_join_room := _s "Join Room";
-  iM_me_prefered_rooms := _s "Prefered Rooms";
+  iM_me_preferred_rooms := _s "Preferred Rooms";
   iM_lb_new_account := _s "New Account";
   im_lb_room_name := _s "Room Name";
   iM_lb_join_room := _s "Join Room";
