@@ -433,35 +433,36 @@ let color_of_name name =
 
 let fit_string_to_pixels s ~context ~pixels =
   let us = U.utf8_of s in
-  let pango_layout = Pango.Layout.create context#as_context in
-  Pango.Layout.set_text pango_layout us;
-  let us' =
-    if fst (Pango.Layout.get_pixel_size pango_layout) <= pixels
+  if us = ""
     then us
     else begin
-      let len = Glib.Utf8.length us in
-      let uca = Glib.Utf8.to_unistring us in
-      let uca_ext = Array.sub uca (len - 7) 7 in
-      let ext = "..." ^ (Glib.Utf8.from_unistring uca_ext) in
-      let rec adjust_string n len =
-        if n = len
-          then us
-          else begin
-            let ustr = (Glib.Utf8.from_unistring (Array.sub uca 0 n)) ^ ext in
-            Pango.Layout.set_text pango_layout ustr;
-            let ustr_pixels = fst (Pango.Layout.get_pixel_size pango_layout) in
-            (* Printf.printf "width %d - Pango.Layout.get_pixel_size %d Pango.Layout.get_text %s\n" 
-               pixels s_width (Pango.Layout.get_text pango_layout);
-            flush stdout; *)
-            if ustr_pixels < pixels
-              then adjust_string (n + 1) len
-              else ustr
-          end
-      in
-      adjust_string 0 len
+      let pango_layout = Pango.Layout.create context#as_context in
+      Pango.Layout.set_text pango_layout us;
+      let us' =
+        if fst (Pango.Layout.get_pixel_size pango_layout) <= pixels
+        then us
+        else begin
+          let len = Glib.Utf8.length us in
+          let uca = Glib.Utf8.to_unistring us in
+          let ext_len = min 7 (len - 1) in
+          let uca_ext = Array.sub uca (len - ext_len) ext_len in
+          let ext = "..." ^ (Glib.Utf8.from_unistring uca_ext) in
+          let rec adjust_string n len =
+            if n = len
+              then us
+              else begin
+                let ustr = (Glib.Utf8.from_unistring (Array.sub uca 0 n)) ^ ext in
+                Pango.Layout.set_text pango_layout ustr;
+                let ustr_pixels = fst (Pango.Layout.get_pixel_size pango_layout) in
+                if ustr_pixels < pixels
+                  then adjust_string (n + 1) len
+                  else ustr
+              end
+          in
+          adjust_string 0 len
+        end
+      in us'
     end
-  in us'
-
 
 
 let warning_box
@@ -493,18 +494,18 @@ let warning_box
       ~packing:(vbox#pack ~expand:false ~fill:true) ()
   in
   let pixbuf = A.get_icon ~icon:M.icon_stock_warning ~size:A.LARGE () in
-  let image =
+  let _image =
     GMisc.image ~pixbuf ~xalign:0. ~yalign:0.
       ~packing:(hbox#pack ~expand:false ~fill:true) ()
   in
   let markup = create_default_bold_markup warning_message in
-  let w_label =
+  let _w_label =
     GMisc.label ~xalign:0.5 ~yalign:0.5 ~markup
       ~packing:(hbox#pack ~expand:true ~fill:true) ()
   in
   if List.length text > 0
     then begin
-      let separator =
+      let _separator =
         GMisc.separator `HORIZONTAL
         ~packing:(vbox#pack ~expand:false ~fill:true) ()
       in
@@ -531,7 +532,7 @@ let warning_box
         incr pos
       ) text
     end;
-  let separator =
+  let _separator =
     GMisc.separator `HORIZONTAL
     ~packing:(vbox#pack ~expand:false ~fill:true) ()
   in
