@@ -150,15 +150,6 @@ let display_graph gui () =
   gui.vbox#add graph;
   gui.current_page <- 8
 
-(*************************************************************************)
-(*                                                                       *)
-(*                         display_im                                    *)
-(*                                                                       *)
-(*************************************************************************)
-
-let display_im gui () =
-  GuiIm.main_window ()
-
 (*************************************************************************)
 (*                                                                       *)
 (*                         display_settings                              *)
