@@ -28,6 +28,7 @@ open CommonGlobals
 
 open DonkeyTypes
 open DonkeyMftp
+open Int64ops
 
 module QueryReplyUdp  = struct
 
@@ -50,31 +51,31 @@ module QueryReplyUdp  = struct
 
     let parse len s =
       let rec iter pos list =
-	if pos < len then
+  if pos < len then
           let file, pos = get_file s pos in
           let pos = pos + 2 in
-	  iter pos (file :: list)
-	else List.rev list
+    iter pos (file :: list)
+  else List.rev list
       in
       iter 1 []
 
     let bprint oc t =
       Printf.bprintf oc "FOUND:\n";
       List.iter (fun t ->
-	Printf.bprintf oc "%s\n" (Md4.to_string t.f_md4);
-	Printf.bprintf oc "%s\n" (Ip.to_string t.f_ip);
-	Printf.bprintf oc "%d\n" t.f_port;
-	Printf.bprintf oc "TAGS:\n";
-	bprint_tags oc t.f_tags;
+  Printf.bprintf oc "%s\n" (Md4.to_string t.f_md4);
+  Printf.bprintf oc "%s\n" (Ip.to_string t.f_ip);
+  Printf.bprintf oc "%d\n" t.f_port;
+  Printf.bprintf oc "TAGS:\n";
+  bprint_tags oc t.f_tags;
          Printf.bprintf oc "\n"
       ) t
 
     let write buf t =
       List.iter (fun file ->
-	buf_md4 buf file.f_md4;
-	buf_ip buf file.f_ip;
-	buf_port buf file.f_port;
-	buf_tags buf file.f_tags names_of_tag
+  buf_md4 buf file.f_md4;
+  buf_ip buf file.f_ip;
+  buf_port buf file.f_port;
+  buf_tags buf file.f_tags names_of_tag
       ) t
 
   end
@@ -108,9 +109,8 @@ module PingServerUdp = struct (* client -> serveur pour identification ? *)
 
     let parse len s =
       try
-	get_uint64_32 s 1(*, get_int8 s 2, get_int8 s 3*)
-      with _ ->
-	Int64.zero
+        get_uint64_32 s 1 (*, get_int8 s 2, get_int8 s 3*)
+      with _ -> 0L
 
     let bprint oc t =
       Printf.bprintf oc "PING %s\n" (Int64.to_string t)
@@ -130,27 +130,34 @@ module PingServerUdp = struct (* client -> serveur pour identification ? *)
 
 module PingServerReplyUdp = struct (* reponse du serveur a 150 *)
 
-    let multiple_getsources = 1
-    let multiple_replies = 2
-
     type t = {
         challenge  : int64;
-        users      : int;
-        files      : int;
-        soft_limit : int option;
-        hard_limit : int option;
-        max_users  : int option;
-        flags      : int option;
+        users      : int64;
+        files      : int64;
+        soft_limit : int64 option;
+        hard_limit : int64 option;
+        max_users  : int64 option;
+        lowid_users : int64 option;
+        get_sources : bool;
+        get_files : bool;
+        newtags : bool;
+        unicode : bool;
+        get_sources2 : bool;
+        largefiles : bool;
+        udp_obfuscation : bool;
+        tcp_obfuscation : bool;
       }
-(*           <E3><97><users><files><softLimit><hardLimit><maxUsers><flags> *)
+
     let parse len s =
       let challenge = get_uint64_32 s 1 in
-      let users = get_int s 5 in
-      let files = get_int s 9 in
-      let max_users  = if len >= 16 then Some (get_int s 13) else None in
-      let soft_limit = if len >= 20 then Some (get_int s 17) else None in
-      let hard_limit = if len >= 24 then Some (get_int s 21) else None in
-      let flags      = if len >= 28 then Some (get_int s 25) else None in
+      let users = get_uint64_32 s 5 in
+      let files = get_uint64_32 s 9 in
+      let max_users  = if len >= 17 then Some (get_uint64_32 s 13) else None in
+      let soft_limit = if len >= 21 then Some (get_uint64_32 s 17) else None in
+      let hard_limit = if len >= 25 then Some (get_uint64_32 s 21) else None in
+      let flags      = if len >= 29 then get_int s 25 else 0 in
+      let lowid_users = if len >= 33 then Some (get_uint64_32 s 29) else None in
+
       {
         challenge = challenge;
         users = users;
@@ -158,56 +165,62 @@ module PingServerReplyUdp = struct (* reponse du serveur a 150 *)
         soft_limit = soft_limit;
         hard_limit = hard_limit;
         max_users = max_users;
-        flags = flags;
+        lowid_users = lowid_users;
+        get_sources = 0x01 land flags = 0x01;
+        get_files = 0x02 land flags = 0x02;
+        newtags = 0x08 land flags = 0x08;
+        unicode = 0x10 land flags = 0x10;
+        get_sources2 = 0x20 land flags = 0x20;
+        largefiles = 0x100 land flags = 0x100;
+        udp_obfuscation = 0x200 land flags = 0x200;
+        tcp_obfuscation = 0x200 land flags = 0x200;
       }
 
     let bprint oc t =
       Printf.bprintf oc "PING REPLY\n";
-      Printf.bprintf oc "   %d users %d files\n" t.users t.files;
-      (match t.soft_limit with Some x -> Printf.bprintf oc "   Soft limit: %d\n" x | None -> ());
-      (match t.hard_limit with Some x -> Printf.bprintf oc "   Hard limit: %d\n" x | None -> ());
-      (match t.max_users with Some x -> Printf.bprintf oc "   Max nusers: %d\n" x | None -> ());
-      (match t.flags with Some x -> Printf.bprintf oc "   Flags: %x\n" x | None -> ());
-      Printf.bprintf oc "\n"
+      Printf.bprintf oc "   %Ld users %Ld files\n" t.users t.files;
+      (match t.soft_limit with Some x -> Printf.bprintf oc "   Soft limit: %Ld\n" x | None -> ());
+      (match t.hard_limit with Some x -> Printf.bprintf oc "   Hard limit: %Ld\n" x | None -> ());
+      (match t.max_users with Some x -> Printf.bprintf oc "   Max nusers: %Ld\n" x | None -> ());
+      (match t.lowid_users with Some x -> Printf.bprintf oc "   LowId nusers: %Ld\n" x | None -> ());
+      Printf.bprintf oc "   get_sources %b, get_files %b, newtags %b, unicode %b, get_sources2 %b, largefiles %b, udp_obfuscation %b, tcp_obfuscation %b"
+        t.get_sources t.get_files t.newtags t.unicode t.get_sources2 t.largefiles t.udp_obfuscation t.tcp_obfuscation
 
     let write buf t =
       buf_int64_32 buf t.challenge;
-      buf_int buf t.users;
-      buf_int buf t.files;
-      (match t.soft_limit, t.hard_limit, t.max_users, t.flags with
-          None, None, None, None -> ()
+      buf_int64_32 buf t.users;
+      buf_int64_32 buf t.files;
+      (match t.soft_limit, t.hard_limit, t.max_users with
+          None, None, None -> ()
         | _ ->
-            buf_int buf (
-              match t.soft_limit with Some x -> x | None -> 0);
-            buf_int buf (
-              match t.hard_limit with Some x -> x | None -> 0);
-            buf_int buf (
-              match t.max_users with Some x -> x | None -> 0);
-            match t.flags with Some x -> buf_int buf x | None -> ()
+            buf_int64_32 buf (
+              match t.soft_limit with Some x -> x | None -> 0L);
+            buf_int64_32 buf (
+              match t.hard_limit with Some x -> x | None -> 0L);
+            buf_int64_32 buf (
+              match t.max_users with Some x -> x | None -> 0L)
       )
   end
 
 module ServerDescUdp = struct
-  type t = {
-    ip : Ip.t;
-  }
+  type t = int64
 
-  let parse len s =
-    try
-      let ip = get_ip s 1 in
-	{
-	  ip = ip
-	}
-    with _ ->
-      {
-	  ip = Ip.null
-      }
+  let invalid_len = Int64.of_int 0xF0FF
+
+  let parse len s = Int64.of_string s
 
   let bprint b t =
-    Printf.bprintf b "ServerDescUdpReq %s\n" (Ip.to_string t.ip)
+    Printf.bprintf b "ServerDescUdpReq\n"
+
+(*
+// eserver 16.45+ supports a new OP_SERVER_DESC_RES answer, if the OP_SERVER_DESC_REQ contains a uint32
+// challenge, the server returns additional info with OP_SERVER_DESC_RES. To properly distinguish the
+// old and new OP_SERVER_DESC_RES answer, the challenge has to be selected carefully. The first 2 bytes
+// of the challenge (in network byte order) MUST NOT be a valid string-len-int16!
+*)
 
   let write buf t =
-    buf_ip buf t.ip
+    buf_int64_32 buf t
 
 end
 
@@ -215,16 +228,64 @@ module ServerDescReplyUdp = struct
   type t = {
     name : string;
     desc : string;
+    tags : tag list;
+    challenge : int64;
   }
 
-  let parse len s =
+  let names_of_tag = [
+    "\001", Field_KNOWN "servername";
+    "\011", Field_KNOWN "description";
+    "\012", Field_KNOWN "ping";
+    "\013", Field_KNOWN "fail";
+    "\014", Field_KNOWN "preference";
+    "\015", Field_KNOWN "port";
+    "\016", Field_KNOWN "ip";
+    "\133", Field_KNOWN "dynip";
+    "\135", Field_KNOWN "maxusers";
+    "\136", Field_KNOWN "softfiles";
+    "\137", Field_KNOWN "hardfiles";
+    "\144", Field_KNOWN "lastping";
+    "\145", Field_KNOWN "version";
+    "\146", Field_KNOWN "udpflags";
+    "\147", Field_KNOWN "auxportslist";
+    "\148", Field_KNOWN "lowidusers";
+  ]
+
+  let parse1 len s challenge =
     let name, pos = get_string s 1 in
     let desc, pos = get_string s pos in
      {
+       tags = [];
        name = name;
        desc = desc;
+       challenge = challenge;
      }
 
+  let parse2 len s challenge =
+    let stags,pos = get_tags s 5 names_of_tag in
+    let name = ref "" in
+    let desc = ref "" in
+    List.iter (fun tag ->
+      match tag with
+      | { tag_name = Field_KNOWN "servername"; tag_value = String v } ->
+            name := v
+      | { tag_name = Field_KNOWN "description"; tag_value = String v } ->
+            desc := v
+      | _ -> ()
+    ) stags;
+    { 
+      tags = stags;
+      name = !name;
+      desc = !desc;
+      challenge = challenge;
+    }
+
+  let parse len s =
+    let challenge = get_uint64_32 s 1 in
+    let test = right64 (left64 challenge 48) 48 in
+    let f = if test = ServerDescUdp.invalid_len then parse2 else parse1 in
+    f len s challenge
+
   let bprint b t =
     Printf.bprintf b  "ServerDescReplyUdpReq\n";
     Printf.bprintf b "name : %s\n" t.name;
@@ -244,14 +305,14 @@ module ServerListUdp = struct
   let parse len s =
     try
       let ip = get_ip s 1 in
-	{
-	  ip = ip;
-	}
+  {
+    ip = ip;
+  }
     with _ ->
       {
-	ip = Ip.null
+  ip = Ip.null
       }
-	
+  
   let bprint b t =
     Printf.bprintf b  "ServerListUdp %s\n" (Ip.to_string t.ip)
 
@@ -265,15 +326,14 @@ module QueryServersReplyUdp = DonkeyProtoServer.QueryServersReply
 module QueryLocationUdp = struct
   open DonkeyProtoServer.QueryLocation
 
-    type file = DonkeyProtoServer.QueryLocation.t
-    type t = file list
+    type t = Md4.t list
 
   let parse len s =
     let rec iter pos list =
       if pos < len then
-	iter (pos+16) (get_md4 s pos :: list)
+  iter (pos+16) (get_md4 s pos :: list)
       else
-	List.rev list
+  List.rev list
     in
     iter 1 []
 
@@ -285,6 +345,30 @@ module QueryLocationUdp = struct
     List.iter (fun md4 -> buf_md4 buf md4) t
 end
 
+module QueryLocationUdpReq2 = struct
+
+  type t = (Md4.t * Int64.t) list
+
+  (* We never parse this anyway, it is outgoing only *)
+  let parse len s =
+    let rec iter pos list =
+      if pos < len then
+  iter (pos+20) ( (get_md4 s pos, get_uint64_32 s (pos+16)) :: list)
+      else
+  List.rev list
+    in
+    iter 1 []
+
+  let bprint b t =
+    Printf.bprintf b "UDP QUERY LOCATIONS2: ";
+    List.iter (fun (md4,size) -> Printf.bprintf b "%s|%Ld " (Md4.to_string md4) size) t;
+    Printf.bprintf b "\n"
+
+  let write buf t =
+    List.iter (fun (md4,size) -> buf_md4 buf md4; buf_int64_32 buf size) t
+
+end
+
 module QueryLocationReplyUdp = struct
   open DonkeyProtoServer.QueryLocationReply
 
@@ -337,8 +421,8 @@ module QueryUdp = DonkeyProtoServer.Query
   let parse len s =
     let rec iter list pos =
       if len > pos then
-	let t, pos = parse_query s pos in
-	iter (t :: list) pos
+  let t, pos = parse_query s pos in
+  iter (t :: list) pos
       else List.rev list
     in
     iter [] 1
@@ -362,6 +446,7 @@ type t =
 | PingServerUdpReq of PingServerUdp.t
 | PingServerReplyUdpReq of PingServerReplyUdp.t
 
+| QueryLocationUdpReq2 of QueryLocationUdpReq2.t
 | QueryLocationUdpReq of QueryLocationUdp.t
 | QueryLocationReplyUdpReq of QueryLocationReplyUdp.t
 
@@ -379,6 +464,7 @@ type t =
 | EmuleReaskAckUdpReq of Md4.t
 | EmuleFileNotFoundUdpReq
 | EmuleQueueFullUdpReq
+| EmulePortTestReq
 
 | UnknownUdpReq of int * string
 
@@ -408,6 +494,7 @@ let parse magic s =
     | 145 -> EmuleReaskAckUdpReq (get_md4 s 1)
 (*    | 146 -> EmuleFileNotFoundUdpReq *)
     | 147 -> EmuleQueueFullUdpReq
+    | 254 -> EmulePortTestReq
 
     | _ -> raise Exit
   with
@@ -424,6 +511,7 @@ let print t =
     | QueryUdpReq t -> QueryUdp.bprint b t
     | QueryMultipleUdpReq t -> QueryUdp.bprint b t
     | QueryReplyUdpReq t -> QueryReplyUdp.bprint b t
+    | QueryLocationUdpReq2 t -> QueryLocationUdpReq2.bprint b t
     | QueryLocationUdpReq t -> QueryLocationUdp.bprint b t
     | QueryLocationReplyUdpReq t
     | FileGroupInfoUdpReq t -> QueryLocationReplyUdp.bprint b t
@@ -447,6 +535,8 @@ let print t =
         Printf.bprintf b "EmuleFileNotFoundUdpReq"
     | EmuleQueueFullUdpReq ->
         Printf.bprintf b "EmuleQueueFullUdpReq"
+    | EmulePortTestReq ->
+        Printf.bprintf b "EmulePortTestReq"
 
     | UnknownUdpReq (magic, s) ->
         Printf.bprintf b "UnknownReq magic %d\n" magic;
@@ -510,6 +600,9 @@ let write buf t =
       | QueryLocationUdpReq t ->
           buf_int8 buf 154;
           QueryLocationUdp.write buf t
+      | QueryLocationUdpReq2 t ->
+          buf_int8 buf 148;
+          QueryLocationUdpReq2.write buf t
       | QueryLocationReplyUdpReq t ->
           buf_int8 buf 155;
           QueryLocationReplyUdp.write buf t
@@ -533,6 +626,14 @@ let write buf t =
           buf_int8 buf 53;
           QueryIDReplyUdp.write buf t
 
+      | EmulePortTestReq ->
+	  buf_int8 buf 2;
+	  buf_int8 buf 0;
+	  buf_int8 buf 0;
+	  buf_int8 buf 0;
+	  buf_int8 buf 0xfe;
+	  buf_int8 buf 0x31
+
       | EmuleQueueFullUdpReq
       | EmuleFileNotFoundUdpReq
       | EmuleReaskAckUdpReq _
