@@ -180,7 +180,7 @@ QAnd (QHasMinVal (CommonUploads.filesize_field, n),q)
             let module M = QueryReply in
             let module C = CommonUploads in
             let replies = ref [] in
-            for i = 0 to mini (Array.length files - 1) 9 do
+            for i = 0 to min (Array.length files - 1) 9 do
               let sh, info = files.(i) in
               let infos = ref [] in
               List.iter (fun uid ->
@@ -254,7 +254,7 @@ QAnd (QHasMinVal (CommonUploads.filesize_field, n),q)
 (* probably XML. print to remember that we should be able to use this
 information. *)
               else
-                  lprintf "info: %s\n" (String.escaped s);
+                if !verbose then lprintf "info: %s\n" (String.escaped s);
           ) f.Q.info;
           
           if !verbose then
