@@ -17,6 +17,9 @@
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *)
 
+val slash : char
+(** character used as a directory separator in file paths *)
+
 val dirname : string -> string 
 (*d [dirname filename] returns the dirname of [filename] after normalization *)
 
@@ -34,6 +37,11 @@ val last_extension : string -> string
 which is the substring after the last "." in [filename] 
   (including the "."). If no extension is present, returns "". *)
 
+val last_extension2 : string -> string
+(*d [last_extension filename] returns the last extension of [filename], 
+which is the substring after the last "." in [filename] 
+  (without the "."). If no extension is present, returns "". *)
+
 val extensions : string -> string list
 (*d [extensions filename] returns the list of extensions (without the dot)
   found at the end of filename. *)
@@ -54,15 +62,28 @@ val to_string : string -> string
 (*d [to_string filename] converts [filename] to a string using registered
 conversion functions. *)
   
-val shorten : int -> string -> string
-  
 (* transform a filename in a list of dirname *)
 val path_of_filename : string -> string list
   
 val basename : string -> string
   
-(* remove invalid chars in a filename, depending on the filesystem *)
-val filesystem_compliant : string -> string -> string
+(* remove invalid chars in a filename, depending on the filesystem, 
+   trim filename length to allowed limit on filesystem *)
+val filesystem_compliant : string -> string -> int -> string
 
-val temp_directory : unit -> string
-  
+(** [temp_file prefix suffix] returns the name of a
+   fresh temporary file in the temporary directory.
+   The base name of the temporary file is formed by concatenating
+   [prefix], then a suitably chosen integer number, then [suffix].
+   The temporary file is created empty, with permissions [0o600]
+   (readable and writable only by the file owner).  The file is
+   guaranteed to be different from any other file that existed when
+   [temp_file] was called.
+*)
+val temp_file : string -> string -> string
+
+(** The name of the temporary directory:
+    The value of the [MLDONKEY_TEMP] environment variable is used,
+    or "mlnet_tmp" if the variable is not set.
+*)
+val temp_dir_name : unit -> string
