@@ -1,3 +1,21 @@
+/* Copyright 2001, 2002 b8_bavard, b8_fee_carabine, INRIA */
+/*
+    This file is part of mldonkey.
+
+    mldonkey is free software; you can redistribute it and/or modify
+    it under the terms of the GNU General Public License as published by
+    the Free Software Foundation; either version 2 of the License, or
+    (at your option) any later version.
+
+    mldonkey is distributed in the hope that it will be useful,
+    but WITHOUT ANY WARRANTY; without even the implied warranty of
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+    GNU General Public License for more details.
+
+    You should have received a copy of the GNU General Public License
+    along with mldonkey; if not, write to the Free Software
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+*/
 
 #include "../../../config/config.h"
 
@@ -30,10 +48,14 @@ struct filedescr {
 #define Fd_val(v) (((struct filedescr *) Data_custom_val(v))->fd.handle)
 #define Socket_val(v) (((struct filedescr *) Data_custom_val(v))->fd.socket)
 
+#define Descr_kind_val(v) (((struct filedescr *) Data_custom_val(v))->kind)
+#define OS_IS_FD(v) (KIND_HANDLE == Descr_kind_val(v))
+
 typedef HANDLE OS_FD;
 typedef SOCKET OS_SOCKET;
 typedef unsigned int uint;
 extern void win32_maperr(unsigned long errcode);
+unsigned char * utf8_to_utf16(const char * str);
 
 /*******************************************************************
 
@@ -48,6 +70,8 @@ extern void win32_maperr(unsigned long errcode);
 #define Fd_val(v) Int_val(v)
 #define Socket_val(v) Int_val(v)
 
+#define OS_IS_FD(v) (Is_long(v))
+
 typedef int OS_FD;
 typedef int OS_SOCKET;
 
@@ -127,10 +151,6 @@ typedef off_t OFF_T;
 extern void unix_error (int errcode, char * cmdname, value arg) Noreturn;
 extern void uerror (char * cmdname, value arg) Noreturn;
 
-static int seek_command_table[] = {
-  SEEK_SET, SEEK_CUR, SEEK_END
-};
-
 extern OFF_T os_lseek(OS_FD fd, OFF_T pos, int dir);
 extern void os_ftruncate(OS_FD fd, OFF_T len, int sparse);
 extern ssize_t os_read(OS_FD fd, char *buf, size_t len);
@@ -140,7 +160,7 @@ extern int64 os_getfilesize(char *path);
 extern void os_set_nonblock(OS_SOCKET fd);
 extern void os_uname(char buf[]);
 extern int os_os_supported();
-
+extern int os_fsync(OS_FD fd);
 
 #define HASH_BUFFER_LEN 131072
 extern unsigned char hash_buffer[HASH_BUFFER_LEN];
