@@ -21,6 +21,10 @@ type t
 
 val verbose : bool ref
 
+val get_max_cache_size : unit -> int
+val set_max_cache_size : int -> unit
+val max_cache_size_default : int
+
 val external_start : unit -> unit
 val external_exit : unit -> unit
 val uname : unit -> string
@@ -42,7 +46,6 @@ val close_all : unit -> unit
 val fds_size : int
 val filename : t -> string
 val rename : t -> string -> unit 
-val max_cache_size : int ref
 val mtime : string -> float
 val mtime64 : t -> float
 val owner : string -> (string * string)
@@ -95,10 +98,13 @@ val bsize : string -> int64 option
 val blocks : string -> int64 option
 val bfree : string -> int64 option
 val bavail : string -> int64 option
-val fnamelen : string -> int64 option
+val fnamelen : string -> int option
 val disktotal : string -> int64 option
 val diskfree : string -> int64 option
 val diskused : string -> int64 option
 val percentused : string -> int option
 val percentfree : string -> int option
 val filesystem : string -> string
+val subfile_tree_map : t -> (string -> int64 -> int64 -> int64-> unit)  ->  unit
+val find_file : t -> int64 -> (string * int64 * int64)
+val find_file_index : t -> int -> (string * int64 * int64)
