@@ -105,7 +105,7 @@ type header =
 
 type options = {
     referer : Url.url option;
-    content_length : int;
+    content_length : int64;
     content_type : string;
     login : string;
     passwd : string;
@@ -166,7 +166,7 @@ let escaped s =
 let default_options = {
 (*    authorization = No_auth;*)
     referer = None;
-    content_length = -1;
+    content_length = -1L;
     content_type = "";
     login = "";
     passwd = "";
@@ -240,8 +240,8 @@ let error_page code from_ip from_port my_ip my_port reason =
 		 | Some Blocked -> Printf.sprintf "IP %s is blocked, its part of the used IP blocklist " from_ip
 		 | _ -> "")
     | "404" -> "Not found", Printf.sprintf "The requested URL %swas not found on this server."
-			      (match reason with Some (Url_not_found url) -> url ^ " " | _ -> "")
-    | _ -> Printf.sprintf "Unknown error %s" code, ""
+			      (match reason with Some (Url_not_found url) -> html_escaped url ^ " " | _ -> "")
+    | _ -> Printf.sprintf "Unknown error %s" (html_escaped code), ""
   in
   let reject_message = Printf.sprintf
 "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\">\n<html>
@@ -251,7 +251,7 @@ let error_page code from_ip from_port my_ip my_port reason =
     Autoconf.current_version my_ip my_port
   in
   Printf.sprintf
-"HTTP/1.1 %s %s\nMLDonkey/%s\nConnection: close
+"HTTP/1.1 %s %s\nServer: MLDonkey/%s\nConnection: close
 Content-Type: text/html; charset=iso-8859-1\nContent-length: %d\r\n"
     code error_text Autoconf.current_version (String.length reject_message), reject_message,
   Printf.sprintf "%s %s" code error_text
@@ -286,7 +286,7 @@ let parse_head sock s =
                   passwd = pswd }
               | "content-length"
                 ->
-                { options with content_length = int_of_string value }
+                { options with content_length = Int64.of_string value }
               | "content-type" ->
                 { options with content_type = value }
               | "host" ->
@@ -828,13 +828,11 @@ let handler config t event =
     (* check here if ip is OK *)
       let from_ip = Ip.of_inet_addr from_ip in
       let ip_is_allowed from_ip =
-	match Ip_set.match_ip config.addrs from_ip with
-	| Some br -> true
-	| None -> false
+	Ip_set.match_ip config.addrs from_ip
       in
       let ip_is_blocked from_ip =
 	if config.use_ip_block_list then
-	  match !Ip.banned from_ip with
+	  match !Ip.banned (from_ip, None) with
             None -> false
            | Some reason -> lprintf_nl "%s:%d blocked: %s"
                (Ip.to_string from_ip) from_port reason; true
